"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.usePropsObservable = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var ui_react_1 = require("@proscom/ui-react");
var react_1 = require("react");
var operators_1 = require("rxjs/operators");
var shallowequal_1 = tslib_1.__importDefault(require("shallowequal"));
/**
 * Converts component props or any other data tied to the React render cycle
 * into an rxjs observable.
 * This observable can then be used to construct rxjs pipelines and subscribe
 * component back to it.
 *
 * Note, that subscribing the same component to this observable returned from
 * this function may lead to infinite rerender cycles if props do not stabilize
 * during re-renders.
 * Props are checked with shallow equality to combat this in most cases.
 *
 * @param props - object which is converted to an observable so that
 *  each time any of the object fields change, observable will emit a new value
 * @param factory - function which gets observable from props and returns
 * @param deps - array of dependencies similar to useMemo, which would
 *  restrict `factory` from re-running on every render
 *
 * @example
 * const filteredList$ = usePropsObservable({ search: props.search }, (props$) => {
 *   return combineLatest(props$, list$).pipe(map((props, list) => {
 *     return list.filter(item => item.indexOf(search) >= 0);
 *   }));
 * }, []);
 *
 * const filteredList = useObservableState(filteredList$);
 */
function usePropsObservable(props, factory, deps) {
    var props$ = ui_react_1.useFactoryRef(function () { return new rxjs_1.BehaviorSubject(props); });
    var result$ = react_1.useMemo(function () {
        var base$ = props$.pipe(operators_1.distinctUntilChanged(shallowequal_1.default));
        if (factory) {
            return factory(base$);
        }
        return base$;
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, tslib_1.__spreadArrays([props$], (deps || [])));
    react_1.useEffect(function () {
        props$.next(props);
    }, [props$, props]);
    return result$;
}
exports.usePropsObservable = usePropsObservable;
//# sourceMappingURL=usePropsObservable.js.map