"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useRequestStore = void 0;
var react_1 = require("react");
var prostore_1 = require("@proscom/prostore");
var ui_react_1 = require("@proscom/ui-react");
var useConnectStore_1 = require("./useConnectStore");
var ProstoreContext_1 = require("./ProstoreContext");
var ProstoreSsrContext_1 = require("./ProstoreSsrContext");
/**
 * Attaches component to the provided store and requests data based on the variables and options
 *
 * @param storeOrName - RequestStore to subscribe to, or its name for it to be injected from the context
 * @param variables - Request variables. If they change, request will be fired again
 * @param options - Additional request options. They are not observed
 * @param hookOptions - Additional hook options:
 *  - checkRequest - Optional function which may be used to override default behavior of determining
 *                   when to fire the request
 */
function useRequestStore(storeOrName, variables, options, hookOptions) {
    if (variables === void 0) { variables = undefined; }
    if (options === void 0) { options = undefined; }
    if (hookOptions === void 0) { hookOptions = {}; }
    var checkRequest = hookOptions.checkRequest || prostore_1.checkRequestState;
    // Inject store from context
    var store = ProstoreContext_1.useContextStore(storeOrName);
    // Local copy of store state
    var _a = react_1.useState(store.state), state = _a[0], setState = _a[1];
    // Subscribe to store changes
    useConnectStore_1.useConnectStore(store, setState);
    // Check if we need to request data
    var queryCheck = checkRequest(state, variables);
    // Memoized callback to request data
    var loadData = ui_react_1.useLatestCallbackRef(function (newOptions) {
        if (newOptions === void 0) { newOptions = undefined; }
        return store.loadData(variables, newOptions || options);
    });
    // Request the data if it is should be requested
    react_1.useEffect(function () {
        if (queryCheck.request) {
            // Error should not happen here, because
            // it is handled inside RequestStore,
            // but we attach the catch listener just in case
            loadData().catch(function (e) { return console.error(e); });
        }
    });
    // Attach ssrContext
    var ssrContext = react_1.useContext(ProstoreSsrContext_1.ProstoreSsrContext);
    // Perform request if server rendering
    if (store.ssrId && ssrContext && ssrContext.isServer && queryCheck.request) {
        loadData().catch(function (e) { return console.error(e); });
    }
    // Return stuff which may be useful in the component
    return { check: queryCheck, state: state, load: loadData, store: store };
}
exports.useRequestStore = useRequestStore;
//# sourceMappingURL=useRequestStore.js.map